#!/usr/bin/python
# H_waveform_envelope_spectrum_distance_network.py
# D. Gibbon, 2021-07-06

import sys, re, glob											# import specialised modules
import numpy as np
import matplotlib.pyplot as plt
import scipy.io.wavfile as wave
from scipy.signal import butter, lfilter, medfilt, hilbert
import scipy.spatial.distance as dist
from graphviz import Graph

spectrummax = 3
distancelimit = 1
formantlimit = 0.7

distancemetrics = [ 'canberra', 'chebyshev', 'cityblock', 'correlation', 'cosine', 'euclidean' ]

if len(sys.argv) > 1:
	wavfiledirectory = sys.argv[1]						# get input filename from command line 
	datasetname = sys.argv[2]
else:
	wavfiledirectory = "DATA/DATA_RT/"
	datasetname = "H_RT"

wavfilelist = sorted(glob.glob(wavfiledirectory+"*.wav"))

namelist = []
rawvaluelist = []
for wavfilename in wavfilelist:								# Collect spectra for all files
	wavfilebase = re.sub(".*/", "", wavfilename)
	wavfilebase = re.sub("-mono-16k.wav","",wavfilebase)
	figurefilename = "PNG/H_%s.png"%wavfilebase				# not immediately relevant here
	csvfilename = "CSV/H_%s.png"%wavfilebase				# not immediately relevant here

	fs, signal = wave.read(wavfilename)						# read sampling frequency and signal

	signallength = len(signal)								# define signal length in bytes
	signalseconds = int(signallength / fs)					# define signal length in seconds
	signal = signal / max(abs(signal))						# normalise signal -1 ... 0 ... 1

	b, a = butter(5, 10 / (0.5 * fs), btype="low")			# define Butterworth filter
	envelope = lfilter(b, a, abs(signal))					# apply filter to create lf envelope
	envelope = envelope / max(envelope)						# normalise envelope 0 ... 1

	specmags = np.abs(np.fft.rfft(envelope))
	specmaglen = len(specmags)
	lfspecmaglen = int(round(spectrummax * specmaglen / (fs / 2)))
	lfspecmags = specmags[1:lfspecmaglen]
	lfspecmags = lfspecmags / max(lfspecmags)				# normalise spectrum magnitudes
	lfspecmags = [ 0 if m <= formantlimit else m for m in lfspecmags ]	# focus on formants
	namelist += [ wavfilebase ]
	rawvaluelist += [ lfspecmags ]

newsize = np.max( [ len(val) for val in rawvaluelist ] )	# Interpolate for equal data lengths
valuelist = []
for val in rawvaluelist:
	size = len(val)
	xloc = np.arange(size)
	new_xloc = np.linspace(0, size, newsize)
	new_data = np.interp(new_xloc, xloc, val)
	valuelist += [ new_data ]
valuelist = np.array(valuelist)

# Create distance networks with different metrics using GraphViz
for distancemetric in distancemetrics:

	print("\nProcessing %s with %s"%(wavfilebase, distancemetric))

	distances = dist.pdist(valuelist, metric=distancemetric)

# print(valuelist)

	dist_square = dist.squareform(distances)
	dist_list = dist_square.reshape(dist_square.shape[0] * dist_square.shape[1])			# reformat
	dist_list = (dist_list - np.min(dist_list)) / (np.max(dist_list) - np.min(dist_list))	# normalise
	dist_square = dist_list.reshape(dist_square.shape)

	for row in dist_square:
		print(row)

	print("	processing GraphViz")
	graphvizfilename="DNETS/"+datasetname+"-graphviz-"+distancemetric
	# dot, neato, fdp, sfdp, twopi und circo
	d = Graph('D', filename=graphvizfilename, engine='dot', format='png')
	d.attr('node', shape='ellipse', fontsize='12', size='6,6', rankdir='LR')
	allcount = 0
	count = 0
	for i in range(0, len(namelist)-1):
		for j in range(i+1, len(namelist)):
			firstname = namelist[i]
			secondname = namelist[j]
			distance = dist_square[i][j]
			allcount += 1
			if distance <= distancelimit:
				count += 1
				d.node(firstname)
				d.node(secondname)
				d.edge(firstname, secondname, label="%.3f"%distance)
			else:
				print(firstname,distance,secondname,"too far apart.")
	d.node(wavfilebase+"\n"+distancemetric + ' distance metric\nn=%d/%d, %s max dist'%(count,allcount,distancelimit), shape='box')
	d.render(graphvizfilename, view=True)		# switch screen view or only save file
	plt.close("all")


